package com.me.asteroids.components;

import com.badlogic.gdx.math.Vector2;
import com.me.common.ecs.Component;

public class VelocityComponent implements Component {

    public Vector2 velocity;
    public float angularVelocity;
    public float maxVelocity;

    public VelocityComponent() {
        this(0f, 0f, 0f);
    }

    public VelocityComponent(Vector2 vector) {
        this(vector, 0f);
    }

    public VelocityComponent(float x, float y) {
        this(x, y, 0f);
    }

    public VelocityComponent(float x, float y, float angularVelocity) {
        this(x, y, angularVelocity, Float.MAX_VALUE);
    }

    public VelocityComponent(float x, float y, float angularVelocity, float maxVelocity) {
        this(new Vector2(x, y), angularVelocity, maxVelocity);
    }

    public VelocityComponent(Vector2 velocity, float angularVelocity) {
        this(velocity, angularVelocity, Float.MAX_VALUE);
    }

    public VelocityComponent(Vector2 velocity, float angularVelocity, float maxVelocity) {
        this.velocity = velocity;
        this.angularVelocity = angularVelocity;
        this.maxVelocity = maxVelocity;
    }

    public float getX() {
        return velocity.x;
    }

    public float getY() {
        return velocity.y;
    }

    public void setX(float x) {
        velocity.x = x;
    }

    public void setY(float y) {
        velocity.y = y;
    }

    public void set(Vector2 vector) {
        set(vector.x, vector.y);
    }

    public void set(float x, float y) {
        velocity.x = x;
        velocity.y = y;
    }

}
