package com.me.asteroids.systems;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.Constants;
import com.me.asteroids.EntityFactory;
import com.me.asteroids.Utils;
import com.me.asteroids.components.VelocityComponent;
import com.me.asteroids.components.model.Model;
import com.me.common.ecs.BaseSystem;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;

import static com.me.asteroids.Components.MODEL;
import static com.me.asteroids.Components.POSITION;
import static com.me.asteroids.Components.VELOCITY;
import static com.me.asteroids.Constants.rand;

public class AsteroidSpawningSystem extends BaseSystem {

    private float asteroidSpawnDelay = 0f;

    public AsteroidSpawningSystem(Engine engine) {
        super(engine);
    }

    private Vector2 getRandomSpawnLocation(float asteroidWidth, float asteroidHeight) {
        float halfWidth = asteroidWidth / 2;
        float halfHeight = asteroidHeight / 2;

        Vector2[] spawnPerimeter = new Vector2[]{
                new Vector2(-halfWidth, -halfHeight),
                new Vector2(-halfWidth, Constants.HEIGHT + halfHeight),
                new Vector2(Constants.WIDTH + halfWidth, Constants.HEIGHT + halfHeight),
                new Vector2(Constants.WIDTH + halfWidth, -halfHeight),
        };

        float perimeterLength = (Constants.WIDTH + Constants.HEIGHT + asteroidWidth + asteroidHeight) * 2;
        float positionAlongPerimeter = rand.nextFloat(0, perimeterLength);

        return Utils.getPointAlongPolygon(spawnPerimeter, positionAlongPerimeter);
    }

    private void spawnAsteroid() {
        Entity asteroid = EntityFactory.createAsteroid(engine);

        Model model = MODEL.get(asteroid).model;
        Rectangle aabb = model.getBoundingBox();

        Vector2 position
                = POSITION.get(asteroid).position
                = getRandomSpawnLocation(aabb.getWidth(), aabb.getHeight());
        model.setPosition(position);

        VelocityComponent velocityComponent = VELOCITY.get(asteroid);
        velocityComponent.velocity = new Vector2().setToRandomDirection().scl(rand.nextFloat(1.875f, 3.125f));
        velocityComponent.angularVelocity = rand.nextFloat(-30, 30);

        asteroid.activate();
    }

    @Override
    public void process(float dt) {
        int asteroidCount = engine.getEntitiesInGroup("ASTEROIDS").size;

        if (asteroidCount++ < Constants.ASTEROID_SPAWN_COUNT && (asteroidSpawnDelay -= dt) <= 0) {
            spawnAsteroid();
            asteroidSpawnDelay = Constants.ASTEROID_SPAWN_DELAY;
        }
    }

}
