package com.me.asteroids.systems;

import com.badlogic.gdx.math.Rectangle;
import com.me.asteroids.EventFactory;
import com.me.asteroids.components.ColliderComponent;
import com.me.asteroids.components.ModelComponent;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

import static com.me.asteroids.Components.MODEL;

public class CollisionSystem extends EntitySystem {

    public CollisionSystem(Engine engine) {
        super(engine, ColliderComponent.class, ModelComponent.class);
    }

    @Override
    public void process(float dt) {
        Entity[] entities = getEntities().items;
        for (int i = 0, n = getEntities().size; i < n-1; i++) {
            Entity entityA = entities[i];
            if (!entityA.isActive()) {
                continue;
            }

            Rectangle aabbA = MODEL.get(entityA).model.getBoundingBox();
            for (int j = i + 1; j < n; j++) {
                Entity entityB = entities[j];
                if (!entityB.isActive()) {
                    continue;
                }

                Rectangle aabbB = MODEL.get(entityB).model.getBoundingBox();
                if (aabbA.overlaps(aabbB)) {
                    engine.callEvent(EventFactory.getNewCollisionEvent(entityA, entityB));
                }
            }
        }
    }

    @Override
    public void processEntity(Entity entity, float dt) {}

}
