package com.me.asteroids.systems;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.Utils;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.asteroids.components.model.Model;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

import static com.me.asteroids.Components.ACCELERATION;
import static com.me.asteroids.Components.MODEL;
import static com.me.asteroids.Components.POSITION;
import static com.me.asteroids.Components.VELOCITY;

public class MovementSystem extends EntitySystem {

    private Vector2 tmp = new Vector2();

    public MovementSystem(Engine engine) {
        super(engine, PositionComponent.class, VelocityComponent.class);
    }

    @Override
    public void processEntity(Entity entity, float dt) {
        PositionComponent positionComponent = POSITION.get(entity);
        VelocityComponent velocityComponent = VELOCITY.get(entity);
        Vector2 velocity = velocityComponent.velocity;

        AccelerationComponent accelComponent = ACCELERATION.get(entity);
        if (accelComponent != null && !accelComponent.acceleration.isZero()) {
            velocity.add(tmp.set(accelComponent.acceleration).scl(dt));
        }

        float maxVelocity = velocityComponent.maxVelocity;
        Vector2 position = positionComponent.position;
        if (!velocity.isZero()) {
            velocity.clamp(0, maxVelocity);
            position.add(tmp.set(velocity).scl(dt));
        }

        float angularVelocity = velocityComponent.angularVelocity;
        if (!MathUtils.isZero(angularVelocity)) {
            positionComponent.rotation = Utils.wrapAngle(positionComponent.rotation + (angularVelocity * dt));
        }

        ModelComponent modelComponent = MODEL.get(entity);
        if (modelComponent != null) {
            Model model = modelComponent.model;
            model.setPosition(position);
            model.setRotation(positionComponent.rotation - 90);
        }
    }

}
